//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "StyleFrm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "RVStyle"
#pragma resource "*.dfm"
TfrmStyle *frmStyle;
//---------------------------------------------------------------------------
__fastcall TfrmStyle::TfrmStyle(TComponent* Owner)
        : TForm(Owner)
{
  Updating = false;
}
//---------------------------------------------------------------------------
// Initialization
void __fastcall TfrmStyle::FormCreate(TObject *Sender)
{
  cmbFont->Items->Assign(Screen->Fonts);
}
//---------------------------------------------------------------------------
// Can Kind of this style template be changed?
// (Yes, if it is not one of main predefined style templates)
bool CanChangeKind(const TRVStyleTemplateName Name)
{
  bool r = Name!="Normal" && Name!="Hyperlink";
  if (!r)
    return r;
  for (int i=1; i<=9; i++)
    if (Name=="heading "+IntToStr(i))
      return false;
  return true;
}
//---------------------------------------------------------------------------
// Returns a set of kinds of style templates that can be parents for a style
// template of the specified Kind
TRVStyleTemplateKinds GetPossibleParentKinds(TRVStyleTemplateKind Kind)
{
  TRVStyleTemplateKinds r = TRVStyleTemplateKinds();
  switch (Kind)
  {
    case rvstkParaText:
    case rvstkPara:
      r << rvstkParaText;
      r << rvstkPara;
      break;
    case rvstkText:
      r << rvstkText;
      break;
  }
  return r;
}
//---------------------------------------------------------------------------
// Displaying the style template (in this and subsequent methods, the style
// template is rvs.StyleTemplates[Index]
void TfrmStyle::DisplayStyleTemplate()
{
  Updating = true;
  TRVStyleTemplate* ST = rvs->StyleTemplates->Items[Index];
  txtName->Text = ST->Name;
  txtName->Enabled = ST->Name!="Normal";
  cmbKind->ItemIndex = (int)ST->Kind;
  cmbKind->Enabled = CanChangeKind(ST->Name) && (!ST->Children || !ST->Children->Count);
  rvs->StyleTemplates->AssignToStrings(cmbParent->Items, true,
    GetPossibleParentKinds(ST->Kind), ST, true);
  cmbParent->Items->Insert(0, "(none)");
  if (ST->Parent)
    cmbParent->ItemIndex = cmbParent->Items->IndexOfObject((TObject*)(ST->ParentId));
  else
    cmbParent->ItemIndex = 0;
  if (ST->Kind != rvstkText)
  {
    TRVStyleTemplateKinds Kinds = TRVStyleTemplateKinds();
    Kinds << rvstkPara;
    Kinds << rvstkParaText;
    rvs->StyleTemplates->AssignToStrings(cmbNext->Items, True, Kinds, NULL, false);
    if (ST->Next)
      cmbNext->ItemIndex = cmbNext->Items->IndexOfObject((TObject*)(ST->NextId));
    else
      cmbNext->ItemIndex = cmbNext->Items->IndexOfObject((TObject*)(ST->Id));
  }
  else
  {
    cmbNext->ItemIndex = -1;
    cmbNext->Enabled = false;
  }
  if (ST->ValidTextProperties.Contains(rvfiFontName))
    cmbFont->Text = ST->TextStyle->FontName;
  else
    cmbFont->Text = "";
  if (ST->ValidTextProperties.Contains(rvfiSize))
    cmbFontSize->Text = IntToStr(ST->TextStyle->Size);
  else
    cmbFontSize->Text = "";
  if (ST->ValidTextProperties.Contains(rvfiBold))
    cbBold->Checked = ST->TextStyle->Style.Contains(fsBold);
  else
    cbBold->State = cbGrayed;
  if (ST->ValidTextProperties.Contains(rvfiItalic))
    cbItalic->Checked = ST->TextStyle->Style.Contains(fsItalic);
  else
    cbItalic->State = cbGrayed;
  if (ST->ValidTextProperties.Contains(rvfiUnderline))
    cbUnderline->Checked = ST->TextStyle->Style.Contains(fsUnderline);
  else
    cbUnderline->State = cbGrayed;
  GroupBox3->Visible = ST->Kind != rvstkText;
  if (ST->Kind != rvstkText)
    if (ST->ValidParaProperties.Contains(rvpiAlignment))
      cmbParaAlignment->ItemIndex = (int)ST->ParaStyle->Alignment+1;
    else
      cmbParaAlignment->ItemIndex = 0;
  Updating = false;
}
//---------------------------------------------------------------------------
// An user changed Kind in the cmbKind
void __fastcall TfrmStyle::cmbKindClick(TObject *Sender)
{
  TRVStyleTemplate* ST = rvs->StyleTemplates->Items[Index];
  ST->Kind = (TRVStyleTemplateKind)cmbKind->ItemIndex;
  if (ST->Kind == rvstkText)
    ST->NextId = -1;
  if (ST->Parent && !GetPossibleParentKinds(ST->Kind).Contains(ST->Parent->Kind))
    ST->ParentId = -1;
  DisplayStyleTemplate();
}
//---------------------------------------------------------------------------
// An user changed Parent in the cmbParent
void __fastcall TfrmStyle::cmbParentClick(TObject *Sender)
{
  TRVStyleTemplate* ST = rvs->StyleTemplates->Items[Index];
  if (cmbParent->ItemIndex<=0)
    ST->ParentId = -1;
  else
    ST->ParentId = (TRVStyleTemplateId)(cmbParent->Items->Objects[cmbParent->ItemIndex]);
  DisplayStyleTemplate();
}
//---------------------------------------------------------------------------
void __fastcall TfrmStyle::cmbNextClick(TObject *Sender)
{
  TRVStyleTemplate* ST = rvs->StyleTemplates->Items[Index];
  TRVStyleTemplateId Id = (TRVStyleTemplateId)(cmbNext->Items->Objects[cmbNext->ItemIndex]);
  if (Id==ST->Id)
    ST->NextId = -1;
  else
    ST->ParentId = Id;
  DisplayStyleTemplate();
}
//---------------------------------------------------------------------------
// An user changed the style template name
void __fastcall TfrmStyle::txtNameChange(TObject *Sender)
{
  rvs->StyleTemplates->Items[Index]->Name = txtName->Text;
}
//---------------------------------------------------------------------------

void __fastcall TfrmStyle::cmbFontOrParaChange(TObject *Sender)
{
  if (!Updating)
    UIToST();
}
//---------------------------------------------------------------------------
// OK button
void __fastcall TfrmStyle::Button1Click(TObject *Sender)
{
  if (txtName->Text.IsEmpty())
  {
    Application->MessageBox("Style name cannot be empty", "Please rename", MB_OK);
    txtName->SetFocus();
    return;
  }
  int idx = rvs->StyleTemplates->FindByName(txtName->Text);
  if (idx>=0 && idx!=Index)
  {
    Application->MessageBox(("Style \""+txtName->Text+"\" already exists").c_str(), "Please rename", MB_OK);
    txtName->SetFocus();
    return;
  }
  ModalResult = mrOk;
}
//---------------------------------------------------------------------------
void SetFontStyle(TCheckBox* cb, TRVFontInfoProperty Prop, TFontStyle Style,
  TRVStyleTemplate* ST)
{
   switch(cb->State)
   {
     case cbUnchecked:
       ST->TextStyle->Style >> Style;
       ST->ValidTextProperties >> Prop;
       break;
     case cbChecked:
       ST->TextStyle->Style >> Style;
       ST->ValidTextProperties >> Prop;
       break;
   }
}
// Assigning StyleTemplate's text and paragraph properties from the form
void TfrmStyle::UIToST()
{
  TRVStyleTemplate* ST = rvs->StyleTemplates->Items[Index];
  // Assigning ST->ValidTextProperties and ST->TextStyle
  if (cbTextReset->Checked)
    ST->ValidTextProperties = TRVFontInfoProperties();
  else
    ST->ValidTextProperties >> rvfiFontName >> rvfiSize >>
      rvfiBold >> rvfiItalic >> rvfiUnderline;
  if (! cmbFont->Text.IsEmpty())
  {
    ST->TextStyle->FontName = cmbFont->Text;
    ST->ValidTextProperties << rvfiFontName;
  }
  if (! cmbFontSize->Text.IsEmpty())
  {
    ST->TextStyle->Size = StrToIntDef(cmbFontSize->Text, 10);
    ST->ValidTextProperties = ST->ValidTextProperties << rvfiSize;
  }
  SetFontStyle(cbBold, rvfiBold, fsBold, ST);
  SetFontStyle(cbItalic, rvfiItalic, fsItalic, ST);
  SetFontStyle(cbUnderline, rvfiUnderline, fsUnderline, ST);
  // Assigning ST->ValidParaProperties and ST->ParaStyle
  if (cbParaReset->Checked)
  {
    TRVParaInfoProperties Props = ST->ValidParaProperties;
    Props >> rvpiOutlineLevel;
    ST->ValidParaProperties -= Props;
  }
  else
    ST->ValidParaProperties >> rvpiAlignment;
  if (cmbParaAlignment->ItemIndex>0)
  {
    ST->ParaStyle->Alignment = (TRVAlignment)(cmbParaAlignment->ItemIndex-1);
    ST->ValidParaProperties << rvpiAlignment;    
  }
}

