//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include "PreviewFrm.h"
//---------------------------------------------------------------------------
#pragma link "RVEdit"
#pragma link "RichView"
#pragma link "RVScroll"
#pragma link "RVStyle"
#pragma link "PtblRV"
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
bool TForm1::IsEmpty(TCustomRichViewEdit* rve)
{
  return rve->ItemCount==0 ||
    (rve->ItemCount==1 && rve->GetItemStyle(0)>=0 && rve->GetItemTextW(0)=="");
}
//---------------------------------------------------------------------------
TRichViewEdit* TForm1::GetHeaderOrFooterRV(TRVHFType HFType, bool IsHeader)
{
  if (IsHeader)
	switch (HFType)
	{
	  case rvhftNormal:    return rveHeader;
	  case rvhftFirstPage: return rveHeader1;
	  case rvhftEvenPages: return rveHeaderEv;
	}
  else
	switch (HFType)
	{
	  case rvhftNormal:    return rveFooter;
	  case rvhftFirstPage: return rveFooter1;
	  case rvhftEvenPages: return rveFooterEv;
	}
  return NULL;
}
//---------------------------------------------------------------------------
void TForm1::PreparePrinting()
{
  RVPrint1->AssignSource(rveMain);

  TRVHFType HFType;

  for (HFType = rvhftNormal; HFType<=rvhftEvenPages; HFType=(TRVHFType)(HFType+1))
	if (!IsEmpty(GetHeaderOrFooterRV(HFType, true)))
	  RVPrint1->SetHeader(GetHeaderOrFooterRV(HFType, true)->RVData, HFType);
  else
	RVPrint1->SetHeader(NULL, HFType);

  for (HFType = rvhftNormal; HFType<=rvhftEvenPages; HFType=(TRVHFType)(HFType+1))
	if (!IsEmpty(GetHeaderOrFooterRV(HFType, false)))
	  RVPrint1->SetFooter(GetHeaderOrFooterRV(HFType, false)->RVData, HFType);
  else
	RVPrint1->SetFooter(NULL, HFType);

  RVPrint1->Margins->Left   = StrToFloatDef(txtLeft->Text,0);
  RVPrint1->Margins->Right  = StrToFloatDef(txtRight->Text,0);
  RVPrint1->Margins->Top    = StrToFloatDef(txtTop->Text,0);
  RVPrint1->Margins->Bottom = StrToFloatDef(txtBottom->Text,0);
  RVPrint1->HeaderY        = StrToFloatDef(txtHeader->Text,0);
  RVPrint1->FooterY        = StrToFloatDef(txtFooter->Text,0);
  RVPrint1->MirrorMargins  = cbMirrorMargins->Checked;
  RVPrint1->TitlePage      = cbTitlePage->Checked;
  RVPrint1->FacingPages    = cbFacingPages->Checked;
  RVPrint1->FormatPages(TRVDisplayOptions());
}
//---------------------------------------------------------------------------
void TForm1::AssignAllHeadersAndFootersForRTF()
{
  TRVHFType HFType;

  for (HFType = rvhftNormal; HFType<=rvhftEvenPages; HFType=(TRVHFType)(HFType+1))
  {
    TRichViewEdit* rve = GetHeaderOrFooterRV(HFType, true);
    rveMain->RTFReadProperties->SetHeader(rve->RVData, HFType);
  }

  for (HFType = rvhftNormal; HFType<=rvhftEvenPages; HFType=(TRVHFType)(HFType+1))
  {
    TRichViewEdit* rve = GetHeaderOrFooterRV(HFType, false);
    rveMain->RTFReadProperties->SetFooter(rve->RVData, HFType);
  }
}
//---------------------------------------------------------------------------
void TForm1::ClearAll()
{
  rveMain->Clear();
  rveMain->DeleteUnusedStyles(true, true, true);

  for (TRVHFType HFType = rvhftNormal; HFType<=rvhftEvenPages; HFType=(TRVHFType)(HFType+1))
  {
    for (bool IsHeader=false; IsHeader<=True; IsHeader++)
    {
      TRichViewEdit* rve = GetHeaderOrFooterRV(HFType, IsHeader);
      rve->Clear();
      rve->DeleteUnusedStyles(true, true, true);
    }
  }
}
//---------------------------------------------------------------------------
void TForm1::FormatAll()
{
  rveMain->Format();

  for (TRVHFType HFType = rvhftNormal; HFType<=rvhftEvenPages; HFType=(TRVHFType)(HFType+1))
  {
    for (bool IsHeader=false; IsHeader<=True; IsHeader++)
    {
      TRichViewEdit* rve = GetHeaderOrFooterRV(HFType, IsHeader);
      rve->Format();
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::New1Click(TObject *Sender)
{
  ClearAll();
  FormatAll();
}
//---------------------------------------------------------------------------

void TForm1::OpenFile(const UnicodeString FileName)
{
  ClearAll();
  rveMain->LoadRTF(FileName);
  FormatAll();

  txtHeader->Text = FloatToStrF(rveMain->DocParameters->HeaderY, ffFixed, 1, 1);
  txtFooter->Text = FloatToStrF(rveMain->DocParameters->FooterY, ffFixed, 1, 1);
  txtLeft->Text   = FloatToStrF(rveMain->DocParameters->LeftMargin, ffFixed, 1, 1);
  txtTop->Text    = FloatToStrF(rveMain->DocParameters->TopMargin, ffFixed, 1, 1);
  txtRight->Text  = FloatToStrF(rveMain->DocParameters->RightMargin, ffFixed, 1, 1);
  txtBottom->Text = FloatToStrF(rveMain->DocParameters->BottomMargin, ffFixed, 1, 1);
  cbMirrorMargins->Checked = rveMain->DocParameters->MirrorMargins;
  cbFacingPages->Checked   = rveMain->DocParameters->FacingPages;
  cbTitlePage->Checked     = rveMain->DocParameters->TitlePage;
}
//---------------------------------------------------------------------------
UnicodeString GetFullFileName(const UnicodeString FileName)
{
  UnicodeString r = ExtractFilePath(Application->ExeName)+FileName;
  if (FileExists(r))
	return r;
  r = ExtractFilePath(Application->ExeName)+"..\\"+FileName;
  if (FileExists(r))
	return r;
  r = ExtractFilePath(Application->ExeName)+"..\\..\\"+FileName;
  if (FileExists(r))
	return r;
  return "";
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  AssignAllHeadersAndFootersForRTF();
  OpenFile(GetFullFileName("doc.rtf"));
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Open1Click(TObject *Sender)
{
  if (OpenDialog1->Execute())
	OpenFile(OpenDialog1->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::PrintPreview1Click(TObject *Sender)
{
  PreparePrinting();
  TfrmPreview* frm = new TfrmPreview(Application);
  frm->rvpp->RVPrint = RVPrint1;
  frm->Button1Click(NULL);
  frm->ShowModal();
  delete frm;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Print1Click(TObject *Sender)
{
  bool PrintIt = psd->Execute();
  if (PrintIt)
  {
	PreparePrinting();
	if (RVPrint1->PagesCount>0)
	  RVPrint1->Print("RichView Edit Demo",1,false);
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Exit1Click(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------