unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, RVScroll, RichView, RVStyle, ComCtrls,
  PtblRV, RVReport;

type
  TForm1 = class(TForm)
    RVStyle1: TRVStyle;
    rvHelper: TRVReportHelper;
    rv: TRichView;
    rgFormat: TRadioGroup;
    cbIgnorePageBreaks: TCheckBox;
    Label1: TLabel;
    cbTransparent: TCheckBox;
    sbPage: TScrollBar;
    lblPage: TLabel;
    txtWidth: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    txtHeight: TEdit;
    Label5: TLabel;
    RVStyle2: TRVStyle;
    cbAutoHeight: TCheckBox;
    btnRecreate: TButton;
    ScrollBox1: TScrollBox;
    Img: TImage;
    procedure FormCreate(Sender: TObject);
    procedure txtWidthorHeightChange(Sender: TObject);
    procedure btnRecreateClick(Sender: TObject);
    procedure sbPageChange(Sender: TObject);
  private
    { Private declarations }
    procedure FormatReportHelper;
    function GetPageMetafile(PageNo, Width, Height: Integer): TMetafile;
    function GetPageBitmap(PageNo, Width, Height: Integer): TBitmap;
    procedure DisplayPage;
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var i, j: Integer;
    Stream: TMemoryStream;
begin
  // generating a document
  for i := 1 to 3 do begin
    rv.AddFmt('Chapter %d', [i], 1, 1);
    if rv.ItemCount>1 then
      rv.PageBreaksBeforeItems[rv.ItemCount-1] := True;
    for j := 1 to 20 do
      rv.AddNL('This is a line of text',0,0);
  end;
  rv.Format;
  // initializing ReportHelper
  rvHelper.RichView.Style := RVStyle2;
  rvHelper.RichView.Color := rv.Color;
  rvHelper.RichView.LeftMargin := rv.LeftMargin;
  rvHelper.RichView.TopMargin := rv.TopMargin;
  rvHelper.RichView.RightMargin := rv.RightMargin;
  rvHelper.RichView.BottomMargin := rv.BottomMargin;
  // copying the document to ReportHelper
  Stream := TMemoryStream.Create;
  try
    rv.SaveRVFToStream(Stream, False);
    Stream.Position := 0;
    rvHelper.RichView.LoadRVFFromStream(Stream);
  finally
    Stream.Free;
  end;
  // reformat and display
  FormatReportHelper;
end;
{------------------------------------------------------------------------------}
function GetEditValue(Edit: TEdit): Integer;
begin
  Result := StrToIntDef(Edit.Text, 100);
  if Result<20 then
    Result := 20;
  Edit.Text := IntToStr(Result);
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormatReportHelper;
var Width, Height: Integer;
begin
  rvHelper.IgnorePageBreaks := cbIgnorePageBreaks.Checked;
  Width := GetEditValue(txtWidth);
  Height := GetEditValue(txtHeight);
  rvHelper.Init(Self.Canvas, Width);
  while rvHelper.FormatNextPage(Height) do;
  btnRecreate.Enabled := False;
  sbPage.Enabled := True;
  sbPage.Max := rvHelper.PagesCount;
  DisplayPage;  
end;
{------------------------------------------------------------------------------}
function TForm1.GetPageMetafile(PageNo, Width, Height: Integer): TMetafile;
var Canvas: TMetafileCanvas;
begin
  Result := TMetafile.Create;
  Result.Width := Width;
  Result.Height := Height;
  Canvas := TMetafileCanvas.Create(Result, 0);
  try
    rvHelper.DrawPage(PageNo, Canvas, True, Height);
  finally
    Canvas.Free;
  end;
end;
{------------------------------------------------------------------------------}
function TForm1.GetPageBitmap(PageNo, Width, Height: Integer): TBitmap;
begin
  Result := TBitmap.Create;
  Result.Width := Width;
  Result.Height := Height;
  rvHelper.DrawPage(PageNo, Result.Canvas, True, Height);
end;
{------------------------------------------------------------------------------}
procedure TForm1.DisplayPage;
var PageNo, Width, Height: Integer;
    Gr: TGraphic;
begin
  PageNo := sbPage.Position;
  rvHelper.TransparentBackground := cbTransparent.Checked;
  Width := GetEditValue(txtWidth);   
  if cbAutoHeight.Checked and (PageNo=rvHelper.PagesCount) then
    Height := rvHelper.GetLastPageHeight
  else
    Height := GetEditValue(txtHeight);
  case rgFormat.ItemIndex of
    0: Gr := GetPageBitmap(PageNo, Width, Height);
    1: Gr := GetPageMetafile(PageNo, Width, Height);
    else Gr := nil;
  end;
  Img.Picture.Graphic := Gr;
  Gr.Free;
  lblPage.Caption := Format('Page %d of %d', [PageNo, rvHelper.PagesCount]);
  lblPage.Visible := True;
end;
{------------------------------------------------------------------------------}


procedure TForm1.txtWidthorHeightChange(Sender: TObject);
begin
  btnRecreate.Enabled := True;
  sbPage.Enabled := False;
  lblPage.Visible := False;
end;

procedure TForm1.btnRecreateClick(Sender: TObject);
begin
  FormatReportHelper;
end;

procedure TForm1.sbPageChange(Sender: TObject);
begin
  DisplayPage;
end;


end.
