unit Unit1;

interface

{$I RV_Defs.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, RVScroll, RichView, RVEdit, ComCtrls, RVStyle, PreviewFrm,
  PtblRV, StdCtrls, CRVData, RVRVData, RVTypes;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    rveMain: TRichViewEdit;
    rveHeader: TRichViewEdit;
    rveFooter: TRichViewEdit;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Print1: TMenuItem;
    Exit1: TMenuItem;
    RVStyle1: TRVStyle;
    RVStyle2: TRVStyle;
    RVStyle3: TRVStyle;
    RVPrint1: TRVPrint;
    N1: TMenuItem;
    N2: TMenuItem;
    PrintPreview1: TMenuItem;
    OpenDialog1: TOpenDialog;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    txtLeft: TEdit;
    Label2: TLabel;
    txtRight: TEdit;
    Label3: TLabel;
    txtTop: TEdit;
    Label4: TLabel;
    txtBottom: TEdit;
    GroupBox2: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    txtHeader: TEdit;
    txtFooter: TEdit;
    New1: TMenuItem;
    PrinterSetupDialog1: TPrinterSetupDialog;
    TabSheet5: TTabSheet;
    TabSheet6: TTabSheet;
    TabSheet7: TTabSheet;
    TabSheet8: TTabSheet;
    cbTitlePage: TCheckBox;
    cbFacingPages: TCheckBox;
    cbMirrorMargins: TCheckBox;
    rveHeader1: TRichViewEdit;
    rveFooter1: TRichViewEdit;
    rveHeaderEv: TRichViewEdit;
    rveFooterEv: TRichViewEdit;
    RVStyle4: TRVStyle;
    RVStyle5: TRVStyle;
    RVStyle6: TRVStyle;
    RVStyle7: TRVStyle;
    procedure PrintPreview1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
  private
    { Private declarations }
    function IsEmpty(rve: TCustomRichViewEdit): Boolean;
    procedure PreparePrinting;
    procedure OpenFile(const FileName: String);
    function GetHeaderOrFooterRV(HFType: TRVHFType;
      IsHeader: Boolean): TRichViewEdit;
    procedure ClearAll;
    procedure FormatAll;
    procedure AssignAllHeadersAndFootersForRTF;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

function TForm1.IsEmpty(rve: TCustomRichViewEdit): Boolean;
begin
  Result := (rve.ItemCount=0) or ((rve.ItemCount=1) and (rve.GetItemStyle(0)>=0) and
    (rve.GetItemText(0)=''));
end;

function TForm1.GetHeaderOrFooterRV(HFType: TRVHFType; IsHeader: Boolean): TRichViewEdit;
begin
  Result := nil;
  if IsHeader then
    case HFType of
      rvhftNormal: Result := rveHeader;
      rvhftFirstPage: Result := rveHeader1;
      rvhftEvenPages: Result := rveHeaderEv;
    end
  else
    case HFType of
      rvhftNormal: Result := rveFooter;
      rvhftFirstPage: Result := rveFooter1;
      rvhftEvenPages: Result := rveFooterEv;
    end
end;

procedure TForm1.PreparePrinting;
var HFType: TRVHFType;
begin
  RVPrint1.AssignSource(rveMain);

  for HFType := Low(TRVHFType) to High(TRVHFType) do
    if not IsEmpty(GetHeaderOrFooterRV(HFType, True)) then
      RVPrint1.SetHeader(GetHeaderOrFooterRV(HFType, True).RVData, HFType)
  else
    RVPrint1.SetHeader(nil, HFType);

  for HFType := Low(TRVHFType) to High(TRVHFType) do
    if not IsEmpty(GetHeaderOrFooterRV(HFType, False)) then
      RVPrint1.SetFooter(GetHeaderOrFooterRV(HFType, False).RVData, HFType)
  else
    RVPrint1.SetFooter(nil, HFType);

  RVPrint1.Margins.Left   := StrToFloatDef(txtLeft.Text,0);
  RVPrint1.Margins.Right  := StrToFloatDef(txtRight.Text,0);
  RVPrint1.Margins.Top    := StrToFloatDef(txtTop.Text,0);
  RVPrint1.Margins.Bottom := StrToFloatDef(txtBottom.Text,0);
  RVPrint1.HeaderY        := StrToFloatDef(txtHeader.Text,0);
  RVPrint1.FooterY        := StrToFloatDef(txtFooter.Text,0);
  RVPrint1.MirrorMargins  := cbMirrorMargins.Checked;
  RVPrint1.TitlePage      := cbTitlePage.Checked;
  RVPrint1.FacingPages    := cbFacingPages.Checked;
  RVPrint1.FormatPages(rvdoALL);
end;

procedure TForm1.AssignAllHeadersAndFootersForRTF;
var HFType: TRVHFType;
    rve: TRichViewEdit;
begin

  for HFType := Low(TRVHFType) to High(TRVHFType) do begin
    rve := GetHeaderOrFooterRV(HFType, True);
    rveMain.RTFReadProperties.SetHeader(rve.RVData, HFType);
  end;

  for HFType := Low(TRVHFType) to High(TRVHFType) do begin
    rve := GetHeaderOrFooterRV(HFType, False);
    rveMain.RTFReadProperties.SetFooter(rve.RVData, HFType);
  end;
end;

procedure TForm1.ClearAll;
var HFType: TRVHFType;
    IsHeader: Boolean;
    rve: TRichViewEdit;
begin
  rveMain.Clear;
  rveMain.DeleteUnusedStyles(True, True, True);

  for HFType := Low(TRVHFType) to High(TRVHFType) do
    for IsHeader := False to True do begin
      rve := GetHeaderOrFooterRV(HFType, IsHeader);
      rve.Clear;
      rve.DeleteUnusedStyles(True, True, True);
    end;
end;

procedure TForm1.FormatAll;
var HFType: TRVHFType;
    IsHeader: Boolean;
    rve: TRichViewEdit;
begin
  rveMain.Format;

  for HFType := Low(TRVHFType) to High(TRVHFType) do
    for IsHeader := False to True do begin
      rve := GetHeaderOrFooterRV(HFType, IsHeader);
      rve.Format;
    end;
end;

procedure TForm1.New1Click(Sender: TObject);
begin
  ClearAll;
  FormatAll;
end;

procedure TForm1.OpenFile(const FileName: String);
begin
  Screen.Cursor := crHourGlass;
  try
    ClearAll;
    rveMain.LoadRTF(FileName);
    FormatAll;
  finally
    Screen.Cursor := crDefault;
  end;

  txtHeader.Text := FloatToStrF(rveMain.DocParameters.HeaderY, ffFixed, 1, 1);
  txtFooter.Text := FloatToStrF(rveMain.DocParameters.FooterY, ffFixed, 1, 1);
  txtLeft.Text := FloatToStrF(rveMain.DocParameters.LeftMargin, ffFixed, 1, 1);
  txtTop.Text := FloatToStrF(rveMain.DocParameters.TopMargin, ffFixed, 1, 1);
  txtRight.Text := FloatToStrF(rveMain.DocParameters.RightMargin, ffFixed, 1, 1);
  txtBottom.Text := FloatToStrF(rveMain.DocParameters.BottomMargin, ffFixed, 1, 1);
  cbMirrorMargins.Checked:= rveMain.DocParameters.MirrorMargins;
  cbFacingPages.Checked := rveMain.DocParameters.FacingPages;
  cbTitlePage.Checked := rveMain.DocParameters.TitlePage;
end;



procedure TForm1.FormCreate(Sender: TObject);
begin
  AssignAllHeadersAndFootersForRTF;
  OpenFile(ExtractFilePath(Application.ExeName)+'doc.rtf');
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
    OpenFile(OpenDialog1.FileName);
end;

procedure TForm1.PrintPreview1Click(Sender: TObject);
var frm: TfrmPreview;
begin
  PreparePrinting;
  frm := TfrmPreview.Create(Application);
  frm.rvpp.RVPrint := RVPrint1;
  frm.Button1Click(nil);
  frm.ShowModal;
  frm.Free;
end;

procedure TForm1.Print1Click(Sender: TObject);
var PrintIt: Boolean;
begin
  {$IFDEF RICHVIEWDEF3}
  PrintIt := PrinterSetupDialog1.Execute;
  {$ELSE}
  PrintIt := True;
  {$ENDIF}
  if PrintIt then begin
    PreparePrinting;
    if RVPrint1.PagesCount>0 then
      RVPrint1.Print('RichView Edit Demo',1,False);
  end;
end;



procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

end.
