unit StyleFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, RVStyle;

type
  TfrmStyle = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    txtName: TEdit;
    Label2: TLabel;
    cmbKind: TComboBox;
    Label3: TLabel;
    cmbParent: TComboBox;
    Label4: TLabel;
    cmbNext: TComboBox;
    GroupBox2: TGroupBox;
    Button1: TButton;
    Button2: TButton;
    Label5: TLabel;
    cmbFont: TComboBox;
    cmbFontSize: TComboBox;
    Label6: TLabel;
    cbTextReset: TCheckBox;
    GroupBox3: TGroupBox;
    cbParaReset: TCheckBox;
    cbBold: TCheckBox;
    cbItalic: TCheckBox;
    cbUnderline: TCheckBox;
    Label7: TLabel;
    cmbParaAlignment: TComboBox;
    rvs: TRVStyle;
    procedure FormCreate(Sender: TObject);
    procedure cmbKindClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure cmbParentClick(Sender: TObject);
    procedure cmbNextClick(Sender: TObject);
    procedure ParaOrFontChange(Sender: TObject);
    procedure txtNameChange(Sender: TObject);
  private
    { Private declarations }
    Updating: Boolean;
    procedure UIToST;
  public
    { Public declarations }
    Index: Integer;    
    procedure DisplayStyleTemplate;
  end;

implementation

{$R *.DFM}

// This form assumes that rvs.StyleTemplates and Index are assigned from outside 
{------------------------------------------------------------------------------}
// Initialization
procedure TfrmStyle.FormCreate(Sender: TObject);
begin
  cmbFont.Items.Assign(Screen.Fonts);
end;
{------------------------------------------------------------------------------}
// Can Kind of this style template be changed?
// (Yes, if it is not one of main predefined style templates)
function CanChangeKind(const Name: TRVStyleTemplateName): Boolean;
var i: Integer;
begin
  Result := (Name<>'Normal') and (Name<>'Hyperlink');
  if not Result then
    exit;
  for i := 1 to 9 do
    if Name='heading '+IntToStr(i) then begin
      Result := False;
      exit;
    end;
end;
{------------------------------------------------------------------------------}
// Returns a set of kinds of style templates that can be parents for a style
// template of the specified Kind
function GetPossibleParentKinds(Kind: TRVStyleTemplateKind): TRVStyleTemplateKinds;
begin
  case Kind of
    rvstkParaText, rvstkPara:
      Result := [rvstkParaText, rvstkPara];
    else
      Result := [rvstkText];
  end;
end;
{------------------------------------------------------------------------------}
// Displaying the style template (in this and subsequent methods, the style
// template is rvs.StyleTemplates[Index]
procedure TfrmStyle.DisplayStyleTemplate;
var ST: TRVStyleTemplate;
begin
  Updating := True;
  try
    ST := rvs.StyleTemplates[Index];
    txtName.Text := ST.Name;
    txtName.Enabled := ST.Name<>'Normal';
    cmbKind.ItemIndex := ord(ST.Kind);
    cmbKind.Enabled := CanChangeKind(ST.Name) and ((ST.Children=nil) or (ST.Children.Count=0));
    rvs.StyleTemplates.AssignToStrings(cmbParent.Items, True,
      GetPossibleParentKinds(ST.Kind), ST, True);
    cmbParent.Items.Insert(0, '(none)');
    if ST.Parent<>nil then
      cmbParent.ItemIndex := cmbParent.Items.IndexOfObject(TObject(ST.ParentId))
    else
      cmbParent.ItemIndex := 0;
    if ST.Kind<>rvstkText then begin
      rvs.StyleTemplates.AssignToStrings(cmbNext.Items, True, [rvstkPara, rvstkParaText], nil, False);
      if ST.Next<>nil then
        cmbNext.ItemIndex := cmbNext.Items.IndexOfObject(TObject(ST.NextId))
      else
        cmbNext.ItemIndex := cmbNext.Items.IndexOfObject(TObject(ST.Id))
      end
    else begin
      cmbNext.ItemIndex := -1;
      cmbNext.Enabled := False;
    end;
    if rvfiFontName in ST.ValidTextProperties then
      cmbFont.Text := ST.TextStyle.FontName
    else
      cmbFont.Text := '';
    if rvfiSize in ST.ValidTextProperties then
      cmbFontSize.Text := IntToStr(ST.TextStyle.Size)
    else
      cmbFontSize.Text := '';
    if rvfiBold in ST.ValidTextProperties then
      cbBold.Checked := fsBold in ST.TextStyle.Style
    else
      cbBold.State := cbGrayed;
    if rvfiItalic in ST.ValidTextProperties then
      cbItalic.Checked := fsItalic in ST.TextStyle.Style
    else
      cbItalic.State := cbGrayed;
    if rvfiUnderline in ST.ValidTextProperties then
      cbUnderline.Checked := fsUnderline in ST.TextStyle.Style
    else
      cbUnderline.State := cbGrayed;
    GroupBox3.Visible := ST.Kind<>rvstkText;
    if ST.Kind<>rvstkText then begin
      if rvpiAlignment in ST.ValidParaProperties then
        cmbParaAlignment.ItemIndex := ord(ST.ParaStyle.Alignment)+1
      else
        cmbParaAlignment.ItemIndex := 0;
    end;
  finally
    Updating := False;
  end;
end;
{------------------------------------------------------------------------------}
// An user changed Kind in the cmbKind
procedure TfrmStyle.cmbKindClick(Sender: TObject);
var ST: TRVStyleTemplate;
begin
  ST := rvs.StyleTemplates[Index];
  ST.Kind := TRVStyleTemplateKind(cmbKind.ItemIndex);
  if ST.Kind = rvstkText then
    ST.NextId := -1;
  if (ST.Parent<>nil) and not (ST.Parent.Kind in GetPossibleParentKinds(ST.Kind)) then
    ST.ParentId := -1;
  DisplayStyleTemplate;
end;
{------------------------------------------------------------------------------}
// An user changed Parent in the cmbParent
procedure TfrmStyle.cmbParentClick(Sender: TObject);
var ST: TRVStyleTemplate;
begin
  ST := rvs.StyleTemplates[Index];
  if cmbParent.ItemIndex<=0 then
    ST.ParentId := -1
  else
    ST.ParentId := TRVStyleTemplateId(cmbParent.Items.Objects[cmbParent.ItemIndex]);
  DisplayStyleTemplate;
end;
{------------------------------------------------------------------------------}
// An user changed Next in the cmbNext
procedure TfrmStyle.cmbNextClick(Sender: TObject);
var ST: TRVStyleTemplate;
    Id: TRVStyleTemplateId;
begin
  ST := rvs.StyleTemplates[Index];
  Id := TRVStyleTemplateId(cmbNext.Items.Objects[cmbNext.ItemIndex]);
  if Id=ST.Id then
    ST.NextId := -1
  else
    ST.ParentId := Id;
  DisplayStyleTemplate;
end;
{------------------------------------------------------------------------------}
// An user changed the style template name
procedure TfrmStyle.txtNameChange(Sender: TObject);
var ST: TRVStyleTemplate;
begin
  ST := rvs.StyleTemplates[Index];
  ST.Name := txtName.Text;
end;
{------------------------------------------------------------------------------}
// An user changed text or paragraph attributes
procedure TfrmStyle.ParaOrFontChange(Sender: TObject);
begin
  if not Updating then
    UIToST;
end;
{------------------------------------------------------------------------------}
// OK button
procedure TfrmStyle.Button1Click(Sender: TObject);
var idx: Integer;
begin
  if txtName.Text='' then begin
    Application.MessageBox('Style name cannot be empty', 'Please rename', MB_OK);
    txtName.SetFocus;
    exit;
  end;
  idx := rvs.StyleTemplates.FindByName(txtName.Text);
  if (idx>=0) and (idx<>Index) then begin
    Application.MessageBox(PChar('Style "'+txtName.Text+'" already exists'), 'Please rename', MB_OK);
    txtName.SetFocus;
    exit;
  end;
  ModalResult := mrOk;
end;
{------------------------------------------------------------------------------}
// Assigning StyleTemplate's text and paragraph properties from the form
procedure TfrmStyle.UIToST;
var ST: TRVStyleTemplate;
   {...............................................}
   procedure SetFontStyle(cb: TCheckBox; Prop: TRVFontInfoProperty; Style: TFontStyle);
   begin
     case cb.State of
       cbUnchecked:
         begin
           ST.TextStyle.Style := ST.TextStyle.Style - [Style];
           ST.ValidTextProperties := ST.ValidTextProperties + [Prop];
         end;
       cbChecked:
         begin
           ST.TextStyle.Style := ST.TextStyle.Style + [Style];
           ST.ValidTextProperties := ST.ValidTextProperties + [Prop];
         end;
     end;
   end;
   {...............................................}
begin
  ST := rvs.StyleTemplates[Index];
  // Assigning ST.ValidTextProperties and ST.TextStyle
  if cbTextReset.Checked then
    ST.ValidTextProperties := []
  else
    ST.ValidTextProperties := ST.ValidTextProperties - [rvfiFontName, rvfiSize,
      rvfiBold, rvfiItalic, rvfiUnderline];
  if cmbFont.Text<>'' then begin
    ST.TextStyle.FontName := cmbFont.Text;
    ST.ValidTextProperties := ST.ValidTextProperties + [rvfiFontName];
  end;
  if cmbFontSize.Text<>'' then begin
    ST.TextStyle.Size := StrToIntDef(cmbFontSize.Text, 10);
    ST.ValidTextProperties := ST.ValidTextProperties + [rvfiSize];
  end;
  SetFontStyle(cbBold, rvfiBold, fsBold);
  SetFontStyle(cbItalic, rvfiItalic, fsItalic);
  SetFontStyle(cbUnderline, rvfiUnderline, fsUnderline);
  // Assigning ST.ValidParaProperties and ST.ParaStyle
  if cbParaReset.Checked then
    ST.ValidParaProperties := ST.ValidParaProperties-(RVAllParaInfoProperties-[rvpiOutlineLevel])
  else
    ST.ValidParaProperties := ST.ValidParaProperties - [rvpiAlignment];
  if cmbParaAlignment.ItemIndex>0 then begin
    ST.ParaStyle.Alignment := TRVAlignment(cmbParaAlignment.ItemIndex-1);
    ST.ValidParaProperties := ST.ValidParaProperties + [rvpiAlignment];    
  end;
end;


end.
