unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, ExtCtrls, Menus,
  RVStyle, RVScroll, RichView, RVEdit, XPMan;

{==============================================================================}
{ RichEditor Demo with StyleTemplates
  Menu items disabling/enabling is not implemented here.

  This demo shows all main operations with StyleTemplates:
  - loading default style templates in File | New;
  - displaying information about formatting at the position of the caret;
  - implementing a style combo-box;
  - clear format command;
  - editing styles.

  For simplification, this demo supports a very limited set of text and paragraph attributes.

  IMPORTANT: If you right click the editor in design time, choose "Settings"
  in the context menu, you'll see that the radiogroup is in state
  "Allow adding styles dynamically + style templates"
{==============================================================================}

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    rve: TRichViewEdit;
    rvs: TRVStyle;
    cmbFont: TComboBox;
    btnBold: TSpeedButton;
    btnItalic: TSpeedButton;
    btnUnderline: TSpeedButton;
    btnLeft: TSpeedButton;
    btnCenter: TSpeedButton;
    btnRight: TSpeedButton;
    btnJustify: TSpeedButton;
    btnOpen: TSpeedButton;
    btnSave: TSpeedButton;
    btnSaveAs: TSpeedButton;
    btnNew: TSpeedButton;
    cmbFontSize: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    mitOpen: TMenuItem;
    mitSave: TMenuItem;
    mitNew: TMenuItem;
    mitSaveAs: TMenuItem;
    N1: TMenuItem;
    mitExit: TMenuItem;
    Edit1: TMenuItem;
    mitUndo: TMenuItem;
    mitRedo: TMenuItem;
    N2: TMenuItem;
    mitCut: TMenuItem;
    mitCopy: TMenuItem;
    mitPaste: TMenuItem;
    mitDelete: TMenuItem;
    od: TOpenDialog;
    sd: TSaveDialog;
    cmbStyles: TComboBox;
    Label1: TLabel;
    rvInfo: TRichView;
    Splitter1: TSplitter;
    rvsInfo: TRVStyle;
    btnStyles: TSpeedButton;
    btnClearFormat: TSpeedButton;
    XPManifest1: TXPManifest;
    procedure mitNewClick(Sender: TObject);
    procedure mitOpenClick(Sender: TObject);
    procedure mitSaveClick(Sender: TObject);
    procedure mitSaveAsClick(Sender: TObject);
    procedure mitExitClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure rveCurTextStyleChanged(Sender: TObject);
    procedure rveCurParaStyleChanged(Sender: TObject);
    procedure cmbFontClick(Sender: TObject);
    procedure rveStyleConversion(Sender: TCustomRichViewEdit; StyleNo,
      UserData: Integer; AppliedToText: Boolean; var NewStyleNo: Integer);
    procedure mitUndoClick(Sender: TObject);
    procedure mitRedoClick(Sender: TObject);
    procedure mitCutClick(Sender: TObject);
    procedure mitCopyClick(Sender: TObject);
    procedure mitPasteClick(Sender: TObject);
    procedure mitDeleteClick(Sender: TObject);
    procedure btnApplyParaClick(Sender: TObject);
    procedure cmbFontSizeClick(Sender: TObject);
    procedure cmbFontSizeKeyPress(Sender: TObject; var Key: Char);
    procedure cmbFontSizeExit(Sender: TObject);
    procedure FontStyleButtonClick(Sender: TObject);
    procedure rveParaStyleConversion(Sender: TCustomRichViewEdit; StyleNo,
      UserData: Integer; AppliedToText: Boolean; var NewStyleNo: Integer);
    procedure rveStyleTemplatesChange(Sender: TObject);
    procedure cmbStylesClick(Sender: TObject);
    procedure btnClearFormatClick(Sender: TObject);
    procedure btnStylesClick(Sender: TObject);
    procedure rveChange(Sender: TObject);
  private
    { Private declarations }
    FileName, FontName: String;
    IgnoreChanges: Boolean;
    FontSize: Integer;
    function SaveIfNeeded: Boolean;
    function Save: Boolean;
    function SaveAs: Boolean;
    procedure Open;
    procedure New;
    function GetAlignmentFromUI: TRVAlignment;
    procedure SetAlignmentToUI(Alignment: TRVAlignment);
    procedure SelectInStyleComboBox;
    procedure DescribeFormatting;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses StyleListFrm;

// Parameters for ApplyStyleConversion
const
  TEXT_BOLD       = 1;
  TEXT_ITALIC     = 2;
  TEXT_UNDERLINE  = 3;
  TEXT_APPLYFONTNAME  = 4;
  TEXT_APPLYFONTSIZE  = 5;
// Parameters for ApplyParaStyleConversion
  PARA_ALIGNMENT  = 1;

{$R *.DFM}
{------------------------------------------------------------------------------}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  // Filling font names combobox
  cmbFont.Items.Assign(Screen.Fonts);
  New;
end;
{------------------------------------------------------------------------------}
// current text style was changed
procedure TfrmMain.rveCurTextStyleChanged(Sender: TObject);
var fi: TFontInfo;
begin
  IgnoreChanges := True;
  // Changing selection in comboboxes with font names and sizes:
  fi := rvs.TextStyles[rve.CurTextStyleNo];
  cmbFont.ItemIndex := cmbFont.Items.IndexOf(fi.FontName);
  cmbFontSize.Text := IntToStr(fi.Size);
  // Checking font buttons
  btnBold.Down      := fsBold      in fi.Style;
  btnItalic.Down    := fsItalic    in fi.Style;
  btnUnderline.Down := fsUnderline in fi.Style;
  // Styles
  SelectInStyleComboBox;
  DescribeFormatting;
  IgnoreChanges := False;
end;
{------------------------------------------------------------------------------}
// current paragraph style was changed
procedure TfrmMain.rveCurParaStyleChanged(Sender: TObject);
begin
  SetAlignmentToUI(rvs.ParaStyles[rve.CurParaStyleNo].Alignment);
  SelectInStyleComboBox;
  DescribeFormatting;  
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.rveChange(Sender: TObject);
begin
  DescribeFormatting;
end;
{------------------------------------------------------------------------------}
function TfrmMain.GetAlignmentFromUI: TRVAlignment;
begin
  if btnLeft.Down then
    Result := rvaLeft
  else if btnRight.Down then
    Result := rvaRight
  else if btnCenter.Down then
    Result := rvaCenter
  else
    Result := rvaJustify;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.SetAlignmentToUI(Alignment: TRVAlignment);
begin
  case Alignment of
    rvaLeft:
      btnLeft.Down := True;
    rvaCenter:
      btnCenter.Down := True;
    rvaRight:
      btnRight.Down := True;
    rvaJustify:
      btnJustify.Down := True;
  end;
end;
{------------------------------------------------------------------------------}
// applying font name
procedure TfrmMain.cmbFontClick(Sender: TObject);
begin
  if (cmbFont.ItemIndex<>-1) then begin
    if not IgnoreChanges then begin
      FontName := cmbFont.Items[cmbFont.ItemIndex];
      rve.ApplyStyleConversion(TEXT_APPLYFONTNAME);
    end;
  end;
  if Visible then
    rve.SetFocus;
end;
{------------------------------------------------------------------------------}
// applying font size
procedure TfrmMain.cmbFontSizeClick(Sender: TObject);
begin
  if (cmbFontSize.Text<>'') and not IgnoreChanges then begin
      FontSize := StrToIntDef(cmbFontSize.Text, 10);
      rve.ApplyStyleConversion(TEXT_APPLYFONTSIZE);
  end;
  if Visible then
    rve.SetFocus;
end;
{------------------------------------------------------------------------------}
// bold, italic, underline
procedure TfrmMain.FontStyleButtonClick(Sender: TObject);
var Button: TSpeedButton;
begin
  Button := Sender as TSpeedButton;
  // constants TEXT_BOLD, TEXT_ITALIC and TEXT_UNDERLINE are
  // assigned to the tags of corresponding buttons
  rve.ApplyStyleConversion(Button.Tag);
end;
{------------------------------------------------------------------------------}
// rve.OnStyleConversion
procedure TfrmMain.rveStyleConversion(Sender: TCustomRichViewEdit; StyleNo,
  UserData: Integer; AppliedToText: Boolean; var NewStyleNo: Integer);
var FontInfo: TFontInfo;
begin
  FontInfo := TFontInfo.Create(nil);
  try
    FontInfo.Assign(rvs.TextStyles[StyleNo]);
    case UserData of
      TEXT_BOLD:
        if btnBold.Down then
          FontInfo.Style := FontInfo.Style+[fsBold]
        else
          FontInfo.Style := FontInfo.Style-[fsBold];
      TEXT_ITALIC:
        if btnItalic.Down then
          FontInfo.Style := FontInfo.Style+[fsItalic]
        else
          FontInfo.Style := FontInfo.Style-[fsItalic];
      TEXT_UNDERLINE:
        if btnUnderline.Down then
          FontInfo.Style := FontInfo.Style+[fsUnderline]
        else
          FontInfo.Style := FontInfo.Style-[fsUnderline];
      TEXT_APPLYFONTNAME:
        FontInfo.FontName := FontName;
      TEXT_APPLYFONTSIZE:
        FontInfo.Size     := FontSize;
      // add your code here....
    end;
    NewStyleNo := rvs.FindTextStyle(FontInfo);
  finally
    FontInfo.Free;
  end;
end;
{------------------------------------------------------------------------------}
// The same for paragraph styles
procedure TfrmMain.rveParaStyleConversion(Sender: TCustomRichViewEdit;
  StyleNo, UserData: Integer; AppliedToText: Boolean;
  var NewStyleNo: Integer);
var ParaInfo: TParaInfo;
begin
  ParaInfo := TParaInfo.Create(nil);
  try
    ParaInfo.Assign(rvs.ParaStyles[StyleNo]);
    case UserData of
      PARA_ALIGNMENT:
        ParaInfo.Alignment := GetAlignmentFromUI;
      // add your code here....
    end;
    NewStyleNo := rvs.FindParaStyle(ParaInfo);
  finally
    ParaInfo.Free;
  end;
end;
{------------------------------------------------------------------------------}
// applying paragraph alignment
procedure TfrmMain.btnApplyParaClick(Sender: TObject);
begin
  rve.ApplyParaStyleConversion(PARA_ALIGNMENT);
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.cmbFontSizeKeyPress(Sender: TObject; var Key: Char);
begin
  if ord(Key)=VK_RETURN then begin
    Key := #0;
    cmbFontSizeClick(nil);
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.cmbFontSizeExit(Sender: TObject);
begin
  cmbFontSizeClick(nil);
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitUndoClick(Sender: TObject);
begin
  rve.Undo;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitRedoClick(Sender: TObject);
begin
  rve.Redo;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitCutClick(Sender: TObject);
begin
  rve.CutDef;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitCopyClick(Sender: TObject);
begin
  rve.CopyDef;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitPasteClick(Sender: TObject);
begin
  rve.Paste;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitDeleteClick(Sender: TObject);
begin
  rve.DeleteSelection;
end;
{------------------------------------------------------------------------------}
function TfrmMain.SaveIfNeeded: Boolean;
begin
  Result := True;
  if rve.Modified then
    case Application.MessageBox('Save file now?','File was modified',
                                MB_ICONQUESTION or MB_YESNOCANCEL) of
      IDYES:
        Result := Save;
      IDNO:
        Result := True;
      IDCANCEL:
        Result := False;
    end;
end;
{------------------------------------------------------------------------------}
function TfrmMain.Save: Boolean;
begin
  if FileName='' then
    Result := SaveAs
  else begin
    rve.SaveRVF(FileName, False);
    rve.Modified := False;
    Result := True;
  end;
end;
{------------------------------------------------------------------------------}
function TfrmMain.SaveAs: Boolean;
begin
  if sd.Execute then begin
    FileName := sd.FileName;
    Result := Save;
    if Result then
      Caption := ExtractFileName(FileName) + '- RDemo';
    end
  else
    Result := False;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.Open;
begin
  if not SaveIfNeeded then exit;
  rve.Modified := False;
  New;
  if od.Execute then begin
    FileName := od.FileName;
    rve.LoadRVF(FileName);
    rve.Format;
    rveCurTextStyleChanged(nil);
    rveCurParaStyleChanged(nil);
    Caption := ExtractFileName(FileName) + '- RDemo';
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.New;
begin
  if not SaveIfNeeded then exit;
  FileName := '';
  Caption := 'Unnamed - RDemo';
  rve.LeftMargin := 5;
  rve.RightMargin := 5;
  rve.TopMargin := 5;
  rve.BottomMargin := 5;
  rve.BackgroundBitmap := nil;
  rve.BackgroundStyle := bsNoBitmap;
  rve.Clear;
  // loading default style templates
  rvs.StyleTemplates.LoadFromRVST(
    ExtractFilePath(Application.ExeName)+'Default.rvst', rvs.Units);
  // creating default text and paragraph styles
  rvs.TextStyles.Clear;
  rvs.ParaStyles.Clear;
  rvs.ListStyles.Clear;
  rvs.ParaStyles.Add;
  rvs.TextStyles.Add;
  // formatting them according to "Normal" style template
  if rvs.StyleTemplates.NormalStyleTemplate<>nil then begin
    rvs.StyleTemplates.NormalStyleTemplate.ApplyToParaStyle(rvs.ParaStyles[0]);
    TRVStyleTemplate(nil).ApplyToTextStyle(rvs.TextStyles[0], rvs.StyleTemplates.NormalStyleTemplate);
    rvs.TextStyles[0].ParaStyleTemplateId := rvs.StyleTemplates.NormalStyleTemplate.Id;
  end;
  rve.Format;
  rveStyleTemplatesChange(rve);
  rveCurTextStyleChanged(nil);
  rveCurParaStyleChanged(nil);
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitNewClick(Sender: TObject);
begin
  New;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitOpenClick(Sender: TObject);
begin
  Open;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitSaveClick(Sender: TObject);
begin
  Save;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitSaveAsClick(Sender: TObject);
begin
  SaveAs;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.mitExitClick(Sender: TObject);
begin
  Close;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := SaveIfNeeded;
end;
{============================ Style Templates =================================}
// rve.OnStyleTemplatesChange: filling cmbStyles combo-box
procedure TfrmMain.rveStyleTemplatesChange(Sender: TObject);
begin
  rvs.StyleTemplates.AssignToStrings(cmbStyles.Items, True,
    [rvstkText, rvstkPara, rvstkParaText], nil, False);
end;
{------------------------------------------------------------------------------}
// Selecting in cmbStyles according to the current text and paragraph style
procedure TfrmMain.SelectInStyleComboBox;
var Id: TRVStyleTemplateId;
begin
  if rve.ItemCount=0 then
    exit;
  // selecting text style template, if possible
  Id := rvs.TextStyles[rve.CurTextStyleNo].StyleTemplateId;
  if Id>0 then begin
    cmbStyles.ItemIndex := cmbStyles.Items.IndexOfObject(TObject(Id));
    exit;
  end;
  // otherwise, selecting paragraph style template, if possible
  Id := rvs.ParaStyles[rve.CurParaStyleNo].StyleTemplateId;
  if Id>0 then begin
    cmbStyles.ItemIndex := cmbStyles.Items.IndexOfObject(TObject(Id));
    exit;
  end;
  cmbStyles.ItemIndex := -1;
end;
{------------------------------------------------------------------------------}
// Applying the style template selected in cmbStyles
procedure TfrmMain.cmbStylesClick(Sender: TObject);
begin
  if cmbStyles.ItemIndex<0 then
    exit;
  rve.ApplyStyleTemplate(cmbStyles.ItemIndex);
  rve.SetFocus;
end;
{------------------------------------------------------------------------------}
// Clear format
procedure TfrmMain.btnClearFormatClick(Sender: TObject);
begin
  rve.ApplyStyleTemplate(-1);
end;
{------------------------------------------------------------------------------}
// Editing styles
procedure TfrmMain.btnStylesClick(Sender: TObject);
var frm: TfrmStyleList;
begin
  frm := TfrmStyleList.Create(Application);
  frm.lst.Items := cmbStyles.Items;
  frm.lst.ItemIndex := cmbStyles.ItemIndex;
  frm.rvs.StyleTemplates.AssignStyleTemplates(rvs.StyleTemplates, True);
  if frm.ShowModal=mrOk then
    rve.ChangeStyleTemplates(frm.rvs.StyleTemplates);
  frm.Free;
end;
{------------------------------------------------------------------------------}
// Returns a string describing TextInfo. Only properties included in TextProps
// are described.
// (this is only a demo function; it describes only font name, size, bold,
// italic, and underline attributes)
function DescribeTextProps(TextInfo: TCustomRVFontInfo;
  TextProps: TRVFontInfoProperties): String;
    {.........................................}
    function DescribeFontStyle(Prop: TRVFontInfoProperty; Style: TFontStyle;
      const Yes, No: String): String;
    begin
      Result := '';
      if Prop in TextProps then begin
        if Style in TextInfo.Style then
          Result := Yes
        else
          Result := No;
        Exclude(TextProps, Prop);
        Result := Result+'; ';
      end;
    end;
    {.........................................}
begin
  Result := DescribeFontStyle(rvfiBold, fsBold, 'bold', 'not bold')+
    DescribeFontStyle(rvfiItalic, fsItalic, 'italic', 'not italic')+
   DescribeFontStyle(rvfiUnderline, fsUnderline, 'underlined', 'not underlined');
  if rvfiFontName in TextProps then begin
    Result := Result+'font name '+TextInfo.FontName+'; ';
    Exclude(TextProps, rvfiFontName);
  end;
  if rvfiSize in TextProps then begin
    Result := Result+'font size '+IntToStr(TextInfo.Size)+'; ';
    Exclude(TextProps, rvfiSize);
  end;
  if TextProps<>[] then
    Result := Result+'some other attributes;';
  Result := Trim(Result);
  if (Result<>'') and (Result[Length(Result)]=';') then
    Delete(Result, Length(Result), 1);
end;
{------------------------------------------------------------------------------}
// Returns a string describing ParaInfo. Only properties included in ParaProps
// are described.
// (this is only a demo function; it describes only alignment and heading level)
function DescribeParaProps(ParaInfo: TCustomRVParaInfo;
  ParaProps: TRVParaInfoProperties): String;
begin
  Result := '';
  if rvpiAlignment in ParaProps then begin
    case ParaInfo.Alignment of
      rvaLeft: Result := 'left';
      rvaRight: Result := 'right';
      rvaCenter: Result := 'center';
      else Result := 'justify';
    end;
    Result := 'alignment '+Result+'; ';
    Exclude(ParaProps, rvpiAlignment);
  end;
  if rvpiOutlineLevel in ParaProps then begin
    if ParaInfo.OutlineLevel<=0 then
      Result := Result+'outline level: body text; '
    else
      Result := Result+'outline level '+IntToStr(ParaInfo.OutlineLevel)+'; ';
    Exclude(ParaProps, rvpiOutlineLevel);
  end;
  if ParaProps<>[] then
    Result := Result+'some other attributes;';
  Result := Trim(Result);
  if (Result<>'') and (Result[Length(Result)]=';') then
    Delete(Result, Length(Result), 1);
end;
{------------------------------------------------------------------------------}
// Returns a string describing paragraph properties of StyleTemplate and
// its parents. Level must be 0.
function DescribeStyleTemplatePara(StyleTemplate: TRVStyleTemplate;
  Level: Integer): String;
var s: String;
begin
  if StyleTemplate=nil then begin
    Result := 'none';
    exit;
  end;
  Result := '"'+StyleTemplate.Name+'"'#13#10;
  s := DescribeParaProps(StyleTemplate.ParaStyle, StyleTemplate.ValidParaProperties);
  if s<>'' then
    Result := Result+'('+s+'), ';
  if Level=0 then
    Result := Result+'based on '
  else
    Result := Result+'which is based on ';
  if StyleTemplate.Parent=nil then
    Result := Result+'default paragraph attributes'
  else
    Result := Result+DescribeStyleTemplatePara(StyleTemplate.Parent, Level+1);
end;
{------------------------------------------------------------------------------}
// Returns a string describing paragraph properties of StyleTemplate and
// its parents. Level must be 0. ParaStyleTemplate=True when describing a style
// template applied to a paragraph.
function DescribeStyleTemplateText(StyleTemplate: TRVStyleTemplate; Level: Integer;
  ParaStyleTemplate: Boolean): String;
var s: String;
begin
  if StyleTemplate=nil then begin
    Result := 'none';
    exit;
  end;
  Result := '"'+StyleTemplate.Name+'"'#13#10;
  s := DescribeTextProps(StyleTemplate.TextStyle, StyleTemplate.ValidTextProperties);
  if s<>'' then
    Result := Result+'('+s+')';
  if (StyleTemplate.Next=nil) and not ParaStyleTemplate then
    exit;
  if s<>'' then
    Result := Result+', ';
  if Level=0 then
    Result := Result+'based on '
  else
    Result := Result+'which is based on ';
  if StyleTemplate.Parent=nil then
    Result := Result+'default text attributes'
  else
    Result := Result+DescribeStyleTemplateText(StyleTemplate.Parent, Level+1, ParaStyleTemplate);
end;
{------------------------------------------------------------------------------}
// Describing current text and paragraph formatting of rve.
// The description is shown in rvInfo 
procedure TfrmMain.DescribeFormatting;
var Id: TRVStyleTemplateId;
    ParaInfo: TParaInfo;
    TextInfo: TFontInfo;
    ParaStyleTemplate, StyleTemplate: TRVStyleTemplate;
    s: String;
begin
  rvInfo.Clear;
  if rve.ItemCount=0 then begin
    rvInfo.Format;
    exit;
  end;
  // paragraph attributes
  rvInfo.AddNL('Paragraph', 1, 0);
  ParaInfo := rvs.ParaStyles[rve.CurParaStyleNo];
  Id := ParaInfo.StyleTemplateId;
  ParaStyleTemplate := rvs.StyleTemplates.FindItemById(Id);
  ParaStyleTemplate.UpdateModifiedParaStyleProperties(ParaInfo);
  s := DescribeParaProps(ParaInfo, ParaInfo.ModifiedProperties);
  if s<>'' then begin
    rvInfo.AddNL('Inline attributes: ', 2, 0);
    rvInfo.Add(s, 0);
  end;
  rvInfo.AddNL('Style: ', 2, 0);
  rvInfo.AddTextNL(DescribeStyleTemplatePara(ParaStyleTemplate, 0), 0, -1, 0);
  rvInfo.AddBreak;
  // text attributes
  rvInfo.AddNL('Text', 1, 0);
  TextInfo := rvs.TextStyles[rve.CurTextStyleNo];
  Id := TextInfo.StyleTemplateId;
  StyleTemplate := rvs.StyleTemplates.FindItemById(Id);
  StyleTemplate.UpdateModifiedTextStyleProperties(TextInfo, ParaStyleTemplate);
  s := DescribeTextProps(TextInfo, TextInfo.ModifiedProperties);
  if s<>'' then begin
    rvInfo.AddNL('Inline attributes: ', 2, 0);
    rvInfo.Add(s, 0);
  end;
  rvInfo.AddNL('Text style: ', 2, 0);
  rvInfo.AddTextNL(DescribeStyleTemplateText(StyleTemplate, 0, False), 0, -1, 0);
  rvInfo.AddNL('Paragraph style: ', 2, 0);
  rvInfo.AddTextNL(DescribeStyleTemplateText(ParaStyleTemplate, 0, True), 0, -1, 0);
  rvInfo.Format;
end;



end.
